# Generated by Django 2.2.17 on 2021-01-14 09:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_form", "0038_auto_20201224_0920"),
        ("caluma_form", "0038_remove_table_answer_value"),
    ]

    operations = []
