# Generated by Django 2.2.17 on 2020-11-23 14:28

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0036_default_answers")]

    operations = [
        migrations.AlterField(
            model_name="question",
            name="default_answer",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="caluma_form.Answer",
            ),
        )
    ]
