# Generated by Django 2.2.16 on 2020-11-09 06:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0035_historicalanswer_history_question_type")]

    operations = [
        migrations.AddField(
            model_name="historicalquestion",
            name="default_answer",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="caluma_form.Answer",
            ),
        ),
        migrations.AddField(
            model_name="question",
            name="default_answer",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="caluma_form.Answer",
            ),
        ),
        migrations.AlterField(
            model_name="answer",
            name="document",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="answers",
                to="caluma_form.Document",
            ),
        ),
    ]
